﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Problems.DataAnalysis.Views {
  [View("Interval Constraint Detail View")]
  [Content(typeof(IntervalConstraint), true)]
  public partial class IntervalConstraintView : ItemView {

    public new IntervalConstraint Content {
      get { return (IntervalConstraint)base.Content; }
      set { base.Content = value; }
    }

    public IntervalConstraintView() {
      InitializeComponent();
      expressionInput.ReadOnly = true;
      definitionInput.ReadOnly = true;
      derivationInput.ReadOnly = true;
      if (derivationInput.Text == "False")
        numberderivationInput.ReadOnly = true;
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.Changed += new EventHandler(Content_Changed);
    }

    protected override void DeregisterContentEvents() {
      base.DeregisterContentEvents();
      Content.Changed -= new EventHandler(Content_Changed);
    }

    private void Content_Changed(object sender, EventArgs e) {
      UpdateControls();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      UpdateControls();
    }


    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      lowerboundInput.Enabled = Content != null && !Locked;
    }

    private void UpdateControls() {
      if (Content == null) {
        expressionInput.Text = string.Empty;
        definitionInput.Text = string.Empty;
        lowerboundInput.Text = string.Empty;
        upperboundInput.Text = string.Empty;
        lowerCombo.SelectedItem = null;
        lowerCombo.SelectedText = "--Nothing set--";
        upperCombo.SelectedItem = null;
        upperCombo.SelectedText = "--Nothing set--";
      } else {
        expressionInput.Text = Content.Expression;
        definitionInput.Text = Content.Definition;
        lowerboundInput.Text = Content.Interval.LowerBound.ToString(CultureInfo.InvariantCulture);
        upperboundInput.Text = Content.Interval.UpperBound.ToString(CultureInfo.InvariantCulture);
        lowerCombo.SelectedIndex = lowerCombo.FindString(Content.InclusiveLowerBound ? "True" : "False");
        upperCombo.SelectedIndex = upperCombo.FindString(Content.InclusiveUpperBound ? "True" : "False");
        derivationInput.Text = Content.IsDerivation ? "True" : "False";
        variableInput.Text = Content.Variable;
        numberderivationInput.Text = Content.NumberOfDerivation.ToString();
      }
      SetEnabledStateOfControls();
    }

    private void expressionInput_TextChanged(object sender, EventArgs e) {

    }

    private void variableInput_TextChanged(object sender, EventArgs e) {
      
    }

    private void numberderivationInput_TextChanged(object sender, EventArgs e) {
      if (int.TryParse(numberderivationInput.Text, out var derivation)) {
        if (derivation >= 0) { 
          Content.NumberOfDerivation = derivation;
          errorProvider.SetError(numberderivationInput, string.Empty);
        } else {
          errorProvider.SetError(numberderivationInput, "Invalid Input: Derivation must be positive!");
        }
      } else {
        errorProvider.SetError(numberderivationInput, "Invalid Input: Derivation must be an integer!");
      }
    }

    private void lowerboundInput_TextChanged(object sender, EventArgs e) {
      var value = ParseDoubleValue(lowerboundInput.Text, lowerboundInput);
      if (!double.IsNaN(value)) {
        if (value <= Content.Interval.UpperBound) {
          Content.Interval = new Interval(value, Content.Interval.UpperBound);
          errorProvider.SetError(lowerboundInput, string.Empty);
        } else {
          errorProvider.SetError(lowerboundInput, "Invalid Input: Lowerbound must be smaller than Upperbound!");
        }
      }
    }

    private double ParseDoubleValue(string input, Control control) {
      input = input.ToLower();
      switch (input) {
        case "inf.":
        case "+inf.":
          return double.PositiveInfinity;
        case "-inf.":
          return double.NegativeInfinity;
        default: { 
          if (double.TryParse(input, out var value))
            return value;
          else {
            errorProvider.SetError(control, "Invalid Input: Value must be a double!");
            return double.NaN;
          }
        }
      }
    }
  }
}
